package java.math;

import java.util.Arrays;

class MutableBigInteger
{
    int[] value;
    int intLen;
    int offset;
    static final MutableBigInteger ONE;
    static final int KNUTH_POW2_THRESH_LEN = 6;
    static final int KNUTH_POW2_THRESH_ZEROS = 3;
    
    MutableBigInteger() {
        this.offset = 0;
        this.value = new int[1];
        this.intLen = 0;
    }
    
    MutableBigInteger(int n) {
        this.offset = 0;
        this.value = new int[1];
        this.intLen = 1;
        this.value[0] = n;
    }
    
    MutableBigInteger(int[] value) {
        this.offset = 0;
        this.value = value;
        this.intLen = value.length;
    }
    
    MutableBigInteger(BigInteger bigInteger) {
        this.offset = 0;
        this.intLen = bigInteger.mag.length;
        this.value = Arrays.copyOf(bigInteger.mag, this.intLen);
    }
    
    MutableBigInteger(MutableBigInteger mutableBigInteger) {
        this.offset = 0;
        this.intLen = mutableBigInteger.intLen;
        this.value = Arrays.copyOfRange(mutableBigInteger.value, mutableBigInteger.offset, mutableBigInteger.offset + this.intLen);
    }
    
    private void ones(int intLen) {
        if (intLen > this.value.length) {
            this.value = new int[intLen];
        }
        Arrays.fill(this.value, -1);
        this.offset = 0;
        this.intLen = intLen;
    }
    
    private int[] getMagnitudeArray() {
        if (this.offset > 0 || this.value.length != this.intLen) {
            return Arrays.copyOfRange(this.value, this.offset, this.offset + this.intLen);
        }
        return this.value;
    }
    
    private long toLong() {
        assert this.intLen <= 2 : "this MutableBigInteger exceeds the range of long";
        if (this.intLen == 0) {
            return 0L;
        }
        long n = (long)this.value[this.offset] & 0xFFFFFFFFL;
        return (this.intLen == 2) ? (n << 32 | ((long)this.value[this.offset + 1] & 0xFFFFFFFFL)) : n;
    }
    
    BigInteger toBigInteger(int n) {
        if (this.intLen == 0 || n == 0) {
            return BigInteger.ZERO;
        }
        return new BigInteger(this.getMagnitudeArray(), n);
    }
    
    BigInteger toBigInteger() {
        this.normalize();
        return this.toBigInteger(this.isZero() ? 0 : 1);
    }
    
    BigDecimal toBigDecimal(int n, int n2) {
        if (this.intLen == 0 || n == 0) {
            return BigDecimal.zeroValueOf(n2);
        }
        int[] magnitudeArray = this.getMagnitudeArray();
        int length = magnitudeArray.length;
        int n3 = magnitudeArray[0];
        if (length > 2 || (n3 < 0 && length == 2)) {
            return new BigDecimal(new BigInteger(magnitudeArray, n), Long.MIN_VALUE, n2, 0);
        }
        long n4 = (length == 2) ? (((long)magnitudeArray[1] & 0xFFFFFFFFL) | ((long)n3 & 0xFFFFFFFFL) << 32) : ((long)n3 & 0xFFFFFFFFL);
        return BigDecimal.valueOf((n == -1) ? (-n4) : n4, n2);
    }
    
    long toCompactValue(int n) {
        if (this.intLen == 0 || n == 0) {
            return 0L;
        }
        int[] magnitudeArray = this.getMagnitudeArray();
        int length = magnitudeArray.length;
        int n2 = magnitudeArray[0];
        if (length > 2 || (n2 < 0 && length == 2)) {
            return Long.MIN_VALUE;
        }
        long n3 = (length == 2) ? (((long)magnitudeArray[1] & 0xFFFFFFFFL) | ((long)n2 & 0xFFFFFFFFL) << 32) : ((long)n2 & 0xFFFFFFFFL);
        return (n == -1) ? (-n3) : n3;
    }
    
    void clear() {
        int n = 0;
        this.intLen = n;
        this.offset = n;
        for (int i = 0; i < this.value.length; ++i) {
            this.value[i] = 0;
        }
    }
    
    void reset() {
        int n = 0;
        this.intLen = n;
        this.offset = n;
    }
    
    final int compare(MutableBigInteger mutableBigInteger) {
        int intLen = mutableBigInteger.intLen;
        if (this.intLen < intLen) {
            return -1;
        }
        if (this.intLen > intLen) {
            return 1;
        }
        int[] value = mutableBigInteger.value;
        int i = this.offset;
        int offset = mutableBigInteger.offset;
        while (i < this.intLen + this.offset) {
            int n = this.value[i] + Integer.MIN_VALUE;
            int n2 = value[offset] + Integer.MIN_VALUE;
            if (n < n2) {
                return -1;
            }
            if (n > n2) {
                return 1;
            }
            ++i;
            ++offset;
        }
        return 0;
    }
    
    private int compareShifted(MutableBigInteger mutableBigInteger, int n) {
        int intLen = mutableBigInteger.intLen;
        int n2 = this.intLen - n;
        if (n2 < intLen) {
            return -1;
        }
        if (n2 > intLen) {
            return 1;
        }
        int[] value = mutableBigInteger.value;
        int i = this.offset;
        int offset = mutableBigInteger.offset;
        while (i < n2 + this.offset) {
            int n3 = this.value[i] + Integer.MIN_VALUE;
            int n4 = value[offset] + Integer.MIN_VALUE;
            if (n3 < n4) {
                return -1;
            }
            if (n3 > n4) {
                return 1;
            }
            ++i;
            ++offset;
        }
        return 0;
    }
    
    final int compareHalf(MutableBigInteger mutableBigInteger) {
        int intLen = mutableBigInteger.intLen;
        int intLen2 = this.intLen;
        if (intLen2 <= 0) {
            return (intLen <= 0) ? 0 : -1;
        }
        if (intLen2 > intLen) {
            return 1;
        }
        if (intLen2 < intLen - 1) {
            return -1;
        }
        int[] value = mutableBigInteger.value;
        int n = 0;
        int n2 = 0;
        if (intLen2 != intLen) {
            if (value[n] == 1) {
                ++n;
                n2 = Integer.MIN_VALUE;
            }
            else {
                return -1;
            }
        }
        int[] value2 = this.value;
        int i = this.offset;
        while (i < intLen2 + this.offset) {
            int n3 = value[n++];
            long n4 = (long)((n3 >>> 1) + n2) & 0xFFFFFFFFL;
            long n5 = (long)value2[i++] & 0xFFFFFFFFL;
            if (n5 != n4) {
                return (n5 < n4) ? -1 : 1;
            }
            n2 = (n3 & 0x1) << 31;
        }
        return (n2 == 0) ? 0 : -1;
    }
    
    private final int getLowestSetBit() {
        if (this.intLen == 0) {
            return -1;
        }
        int n;
        for (n = this.intLen - 1; n > 0 && this.value[n + this.offset] == 0; --n) {}
        int i = this.value[n + this.offset];
        if (i == 0) {
            return -1;
        }
        return (this.intLen - 1 - n << 5) + Integer.numberOfTrailingZeros(i);
    }
    
    private final int getInt(int n) {
        return this.value[this.offset + n];
    }
    
    private final long getLong(int n) {
        return (long)this.value[this.offset + n] & 0xFFFFFFFFL;
    }
    
    final void normalize() {
        if (this.intLen == 0) {
            this.offset = 0;
            return;
        }
        int offset = this.offset;
        if (this.value[offset] != 0) {
            return;
        }
        while (++offset < offset + this.intLen && this.value[offset] == 0) {}
        int n = offset - this.offset;
        this.intLen -= n;
        this.offset = ((this.intLen == 0) ? 0 : (this.offset + n));
        return;
    }
    
    private final void ensureCapacity(int intLen) {
        if (this.value.length < intLen) {
            this.value = new int[intLen];
            this.offset = 0;
            this.intLen = intLen;
        }
    }
    
    int[] toIntArray() {
        int[] array = new int[this.intLen];
        for (int i = 0; i < this.intLen; ++i) {
            array[i] = this.value[this.offset + i];
        }
        return array;
    }
    
    void setInt(int n, int n2) {
        this.value[this.offset + n] = n2;
    }
    
    void setValue(int[] value, int intLen) {
        this.value = value;
        this.intLen = intLen;
        this.offset = 0;
    }
    
    void copyValue(MutableBigInteger mutableBigInteger) {
        int intLen = mutableBigInteger.intLen;
        if (this.value.length < intLen) {
            this.value = new int[intLen];
        }
        System.arraycopy(mutableBigInteger.value, mutableBigInteger.offset, this.value, 0, intLen);
        this.intLen = intLen;
        this.offset = 0;
    }
    
    void copyValue(int[] array) {
        int length = array.length;
        if (this.value.length < length) {
            this.value = new int[length];
        }
        System.arraycopy(array, 0, this.value, 0, length);
        this.intLen = length;
        this.offset = 0;
    }
    
    boolean isOne() {
        return this.intLen == 1 && this.value[this.offset] == 1;
    }
    
    boolean isZero() {
        return this.intLen == 0;
    }
    
    boolean isEven() {
        return this.intLen == 0 || (this.value[this.offset + this.intLen - 1] & 0x1) == 0x0;
    }
    
    boolean isOdd() {
        return !this.isZero() && (this.value[this.offset + this.intLen - 1] & 0x1) == 0x1;
    }
    
    boolean isNormal() {
        return this.intLen + this.offset <= this.value.length && (this.intLen == 0 || this.value[this.offset] != 0);
    }
    
    @Override
    public String toString() {
        return this.toBigInteger(1).toString();
    }
    
    void safeRightShift(int n) {
        if (n / 32 >= this.intLen) {
            this.reset();
        }
        else {
            this.rightShift(n);
        }
    }
    
    void rightShift(int n) {
        if (this.intLen == 0) {
            return;
        }
        int n2 = n >>> 5;
        int n3 = n & 0x1F;
        this.intLen -= n2;
        if (n3 == 0) {
            return;
        }
        if (n3 >= BigInteger.bitLengthForInt(this.value[this.offset])) {
            this.primitiveLeftShift(32 - n3);
            --this.intLen;
        }
        else {
            this.primitiveRightShift(n3);
        }
        return;
    }
    
    void safeLeftShift(int n) {
        if (n > 0) {
            this.leftShift(n);
        }
    }
    
    void leftShift(int n) {
        if (this.intLen == 0) {
            return;
        }
        int n2 = n >>> 5;
        int n3 = n & 0x1F;
        int bitLengthForInt = BigInteger.bitLengthForInt(this.value[this.offset]);
        if (n <= 32 - bitLengthForInt) {
            this.primitiveLeftShift(n3);
            return;
        }
        int intLen = this.intLen + n2 + 1;
        if (n3 <= 32 - bitLengthForInt) {
            --intLen;
        }
        if (this.value.length < intLen) {
            int[] array = new int[intLen];
            for (int l = 0; l < this.intLen; ++l) {
                array[l] = this.value[this.offset + l];
            }
            this.setValue(array, intLen);
        }
        else if (this.value.length - this.offset >= intLen) {
            for (int k = 0; k < intLen - this.intLen; ++k) {
                this.value[this.offset + this.intLen + k] = 0;
            }
        }
        else {
            for (int i = 0; i < this.intLen; ++i) {
                this.value[i] = this.value[this.offset + i];
            }
            for (int j = this.intLen; j < intLen; ++j) {
                this.value[j] = 0;
            }
            this.offset = 0;
        }
        this.intLen = intLen;
        if (n3 == 0) {
            return;
        }
        if (n3 <= 32 - bitLengthForInt) {
            this.primitiveLeftShift(n3);
        }
        else {
            this.primitiveRightShift(32 - n3);
        }
        return;
    }
    
    private int divadd(int[] array, int[] array2, int n) {
        long n2 = 0L;
        for (int i = array.length - 1; i >= 0; --i) {
            long n3 = ((long)array[i] & 0xFFFFFFFFL) + ((long)array2[i + n] & 0xFFFFFFFFL) + n2;
            array2[i + n] = (int)n3;
            n2 = n3 >>> 32;
        }
        return (int)n2;
    }
    
    private int mulsub(int[] array, int[] array2, int n, int n2, int n3) {
        long n4 = (long)n & 0xFFFFFFFFL;
        long n5 = 0L;
        n3 += n2;
        for (int i = n2 - 1; i >= 0; --i) {
            long n6 = ((long)array2[i] & 0xFFFFFFFFL) * n4 + n5;
            long n7 = (long)array[n3] - n6;
            array[n3--] = (int)n7;
            n5 = (n6 >>> 32) + (long)((n7 & 0xFFFFFFFFL) > ((long)~(int)n6 & 0xFFFFFFFFL));
        }
        return (int)n5;
    }
    
    private int mulsubBorrow(int[] array, int[] array2, int n, int n2, int n3) {
        long n4 = (long)n & 0xFFFFFFFFL;
        long n5 = 0L;
        n3 += n2;
        for (int i = n2 - 1; i >= 0; --i) {
            long n6 = ((long)array2[i] & 0xFFFFFFFFL) * n4 + n5;
            n5 = (n6 >>> 32) + (long)(((long)array[n3--] - n6 & 0xFFFFFFFFL) > ((long)~(int)n6 & 0xFFFFFFFFL));
        }
        return (int)n5;
    }
    
    private final void primitiveRightShift(int n) {
        int[] value = this.value;
        int n2 = 32 - n;
        int i = this.offset + this.intLen - 1;
        int n3 = value[i];
        while (i > this.offset) {
            n3 = value[i - 1];
            value[i] = (n3 << n2 | n3 >>> n);
            --i;
        }
        int offset = this.offset;
        value[offset] >>>= n;
    }
    
    private final void primitiveLeftShift(int n) {
        int[] value = this.value;
        int n2 = 32 - n;
        int i = this.offset;
        int n3 = value[i];
        while (i < i + this.intLen - 1) {
            n3 = value[i + 1];
            value[i] = (n3 << n | n3 >>> n2);
            ++i;
        }
        int n4 = this.offset + this.intLen - 1;
        value[n4] <<= n;
    }
    
    private BigInteger getLower(int n) {
        if (this.isZero()) {
            return BigInteger.ZERO;
        }
        if (this.intLen < n) {
            return this.toBigInteger(1);
        }
        for (; n > 0 && this.value[this.offset + this.intLen - n] == 0; --n) {}
        return new BigInteger(Arrays.copyOfRange(this.value, this.offset + this.intLen - n, this.offset + this.intLen), (int)(n > 0));
    }
    
    private void keepLower(int intLen) {
        if (this.intLen >= intLen) {
            this.offset += this.intLen - intLen;
            this.intLen = intLen;
        }
    }
    
    void add(MutableBigInteger mutableBigInteger) {
        int i = this.intLen;
        int j = mutableBigInteger.intLen;
        int intLen = (this.intLen > mutableBigInteger.intLen) ? this.intLen : mutableBigInteger.intLen;
        int[] value;
        int n;
        long n2;
        long n3;
        for (value = ((this.value.length < intLen) ? new int[intLen] : this.value), n = value.length - 1, n2 = 0L; i > 0 && j > 0; --i, --j, n3 = ((long)this.value[i + this.offset] & 0xFFFFFFFFL) + ((long)mutableBigInteger.value[j + mutableBigInteger.offset] & 0xFFFFFFFFL) + n2, value[n--] = (int)n3, n2 = n3 >>> 32) {}
        while (i > 0) {
            --i;
            if (n2 == 0L && value == this.value && n == i + this.offset) {
                return;
            }
            long n4 = ((long)this.value[i + this.offset] & 0xFFFFFFFFL) + n2;
            value[n--] = (int)n4;
            n2 = n4 >>> 32;
        }
        while (j > 0) {
            --j;
            long n5 = ((long)mutableBigInteger.value[j + mutableBigInteger.offset] & 0xFFFFFFFFL) + n2;
            value[n--] = (int)n5;
            n2 = n5 >>> 32;
        }
        if (n2 > 0L) {
            ++intLen;
            if (value.length < intLen) {
                int[] array = new int[intLen];
                System.arraycopy(value, 0, array, 1, value.length);
                array[0] = 1;
                value = array;
            }
            else {
                value[n--] = 1;
            }
        }
        this.value = value;
        this.intLen = intLen;
        this.offset = value.length - intLen;
    }
    
    void addShifted(MutableBigInteger mutableBigInteger, int n) {
        if (mutableBigInteger.isZero()) {
            return;
        }
        int i;
        int j;
        int[] value;
        int n2;
        long n3;
        long n4;
        for (i = this.intLen, j = mutableBigInteger.intLen + n, j = ((this.intLen > j) ? this.intLen : j), value = ((this.value.length < j) ? new int[j] : this.value), n2 = value.length - 1, n3 = 0L; i > 0 && j > 0; --i, n4 = ((long)this.value[i + this.offset] & 0xFFFFFFFFL) + ((long)((--j + mutableBigInteger.offset < mutableBigInteger.value.length) ? mutableBigInteger.value[j + mutableBigInteger.offset] : 0) & 0xFFFFFFFFL) + n3, value[n2--] = (int)n4, n3 = n4 >>> 32) {}
        while (i > 0) {
            --i;
            if (n3 == 0L && value == this.value && n2 == i + this.offset) {
                return;
            }
            long n5 = ((long)this.value[i + this.offset] & 0xFFFFFFFFL) + n3;
            value[n2--] = (int)n5;
            n3 = n5 >>> 32;
        }
        while (j > 0) {
            long n6 = ((long)((--j + mutableBigInteger.offset < mutableBigInteger.value.length) ? mutableBigInteger.value[j + mutableBigInteger.offset] : 0) & 0xFFFFFFFFL) + n3;
            value[n2--] = (int)n6;
            n3 = n6 >>> 32;
        }
        if (n3 > 0L) {
            ++j;
            if (value.length < j) {
                int[] array = new int[j];
                System.arraycopy(value, 0, array, 1, value.length);
                array[0] = 1;
                value = array;
            }
            else {
                value[n2--] = 1;
            }
        }
        this.value = value;
        this.intLen = j;
        this.offset = value.length - j;
        return;
    }
    
    void addDisjoint(MutableBigInteger mutableBigInteger, int n) {
        if (mutableBigInteger.isZero()) {
            return;
        }
        int intLen = this.intLen;
        int intLen2 = mutableBigInteger.intLen + n;
        intLen2 = ((this.intLen > intLen2) ? this.intLen : intLen2);
        int[] value;
        if (this.value.length < intLen2) {
            value = new int[intLen2];
        }
        else {
            value = this.value;
            Arrays.fill(this.value, this.offset + this.intLen, this.value.length, 0);
        }
        int n2 = value.length - 1;
        System.arraycopy(this.value, this.offset, value, n2 + 1 - intLen, intLen);
        int a = intLen2 - intLen;
        int n3 = n2 - intLen;
        int min = Math.min(a, mutableBigInteger.value.length - mutableBigInteger.offset);
        System.arraycopy(mutableBigInteger.value, mutableBigInteger.offset, value, n3 + 1 - a, min);
        for (int i = n3 + 1 - a + min; i < n3 + 1; ++i) {
            value[i] = 0;
        }
        this.value = value;
        this.intLen = intLen2;
        this.offset = value.length - intLen2;
        return;
    }
    
    void addLower(MutableBigInteger mutableBigInteger, int intLen) {
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger(mutableBigInteger);
        if (mutableBigInteger2.offset + mutableBigInteger2.intLen >= intLen) {
            mutableBigInteger2.offset = mutableBigInteger2.offset + mutableBigInteger2.intLen - intLen;
            mutableBigInteger2.intLen = intLen;
        }
        mutableBigInteger2.normalize();
        this.add(mutableBigInteger2);
    }
    
    int subtract(MutableBigInteger mutableBigInteger) {
        int[] value = this.value;
        int compare = this.compare(mutableBigInteger);
        if (compare == 0) {
            this.reset();
            return 0;
        }
        if (compare < 0) {
            this = (mutableBigInteger = mutableBigInteger);
        }
        int intLen = this.intLen;
        if (value.length < intLen) {
            value = new int[intLen];
        }
        long n;
        int i;
        int j;
        int n2;
        for (n = 0L, i = this.intLen, j = mutableBigInteger.intLen, n2 = value.length - 1; j > 0; --j, n = ((long)this.value[i + this.offset] & 0xFFFFFFFFL) - ((long)mutableBigInteger.value[j + mutableBigInteger.offset] & 0xFFFFFFFFL) - (long)(int)(-(n >> 32)), value[n2--] = (int)n) {
            --i;
        }
        while (i > 0) {
            --i;
            n = ((long)this.value[i + this.offset] & 0xFFFFFFFFL) - (long)(int)(-(n >> 32));
            value[n2--] = (int)n;
        }
        this.value = value;
        this.intLen = intLen;
        this.offset = this.value.length - intLen;
        this.normalize();
        return compare;
    }
    
    private int difference(MutableBigInteger mutableBigInteger) {
        int compare = this.compare(mutableBigInteger);
        if (compare == 0) {
            return 0;
        }
        if (compare < 0) {
            this = (mutableBigInteger = mutableBigInteger);
        }
        long n = 0L;
        int i = this.intLen;
        for (int j = mutableBigInteger.intLen; j > 0; --j, n = ((long)this.value[this.offset + i] & 0xFFFFFFFFL) - ((long)mutableBigInteger.value[mutableBigInteger.offset + j] & 0xFFFFFFFFL) - (long)(int)(-(n >> 32)), this.value[this.offset + i] = (int)n) {
            --i;
        }
        while (i > 0) {
            --i;
            n = ((long)this.value[this.offset + i] & 0xFFFFFFFFL) - (long)(int)(-(n >> 32));
            this.value[this.offset + i] = (int)n;
        }
        this.normalize();
        return compare;
    }
    
    void multiply(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2) {
        int intLen = this.intLen;
        int intLen2 = mutableBigInteger.intLen;
        int intLen3 = intLen + intLen2;
        if (mutableBigInteger2.value.length < intLen3) {
            mutableBigInteger2.value = new int[intLen3];
        }
        mutableBigInteger2.offset = 0;
        mutableBigInteger2.intLen = intLen3;
        long n = 0L;
        for (int i = intLen2 - 1, n2 = intLen2 + intLen - 1; i >= 0; --i, --n2) {
            long n3 = ((long)mutableBigInteger.value[i + mutableBigInteger.offset] & 0xFFFFFFFFL) * ((long)this.value[intLen - 1 + this.offset] & 0xFFFFFFFFL) + n;
            mutableBigInteger2.value[n2] = (int)n3;
            n = n3 >>> 32;
        }
        mutableBigInteger2.value[intLen - 1] = (int)n;
        long n4;
        for (int j = intLen - 2; j >= 0; mutableBigInteger2.value[j] = (int)n4, --j) {
            n4 = 0L;
            for (int k = intLen2 - 1, n5 = intLen2 + j; k >= 0; --k, --n5) {
                long n6 = ((long)mutableBigInteger.value[k + mutableBigInteger.offset] & 0xFFFFFFFFL) * ((long)this.value[j + this.offset] & 0xFFFFFFFFL) + ((long)mutableBigInteger2.value[n5] & 0xFFFFFFFFL) + n4;
                mutableBigInteger2.value[n5] = (int)n6;
                n4 = n6 >>> 32;
            }
        }
        mutableBigInteger2.normalize();
    }
    
    void mul(int n, MutableBigInteger mutableBigInteger) {
        if (n == 1) {
            mutableBigInteger.copyValue(this);
            return;
        }
        if (n == 0) {
            mutableBigInteger.clear();
            return;
        }
        long n2 = (long)n & 0xFFFFFFFFL;
        int[] value = (mutableBigInteger.value.length < this.intLen + 1) ? new int[this.intLen + 1] : mutableBigInteger.value;
        long n3 = 0L;
        for (int i = this.intLen - 1; i >= 0; --i) {
            long n4 = n2 * ((long)this.value[i + this.offset] & 0xFFFFFFFFL) + n3;
            value[i + 1] = (int)n4;
            n3 = n4 >>> 32;
        }
        if (n3 == 0L) {
            mutableBigInteger.offset = 1;
            mutableBigInteger.intLen = this.intLen;
        }
        else {
            mutableBigInteger.offset = 0;
            mutableBigInteger.intLen = this.intLen + 1;
            value[0] = (int)n3;
        }
        mutableBigInteger.value = value;
        return;
    }
    
    int divideOneWord(int i, MutableBigInteger mutableBigInteger) {
        long n = (long)i & 0xFFFFFFFFL;
        if (this.intLen == 1) {
            long n6 = (long)this.value[this.offset] & 0xFFFFFFFFL;
            int n7 = (int)(n6 / n);
            int n8 = (int)(n6 - (long)n7 * n);
            mutableBigInteger.value[0] = n7;
            mutableBigInteger.intLen = (n7 != 0);
            mutableBigInteger.offset = 0;
            return n8;
        }
        if (mutableBigInteger.value.length < this.intLen) {
            mutableBigInteger.value = new int[this.intLen];
        }
        mutableBigInteger.offset = 0;
        mutableBigInteger.intLen = this.intLen;
        int numberOfLeadingZeros = Integer.numberOfLeadingZeros(i);
        int n2 = this.value[this.offset];
        long n3 = (long)n2 & 0xFFFFFFFFL;
        if (n3 < n) {
            mutableBigInteger.value[0] = 0;
        }
        else {
            mutableBigInteger.value[0] = (int)(n3 / n);
            n2 = (int)(n3 - (long)mutableBigInteger.value[0] * n);
            n3 = ((long)n2 & 0xFFFFFFFFL);
        }
        int n5;
        for (int intLen = this.intLen; --intLen > 0; mutableBigInteger.value[this.intLen - intLen] = n5, n3 = ((long)n2 & 0xFFFFFFFFL)) {
            long n4 = n3 << 32 | ((long)this.value[this.offset + this.intLen - intLen] & 0xFFFFFFFFL);
            if (n4 >= 0L) {
                n5 = (int)(n4 / n);
                n2 = (int)(n4 - (long)n5 * n);
            }
            else {
                long divWord = divWord(n4, i);
                n5 = (int)(divWord & 0xFFFFFFFFL);
                n2 = (int)(divWord >>> 32);
            }
        }
        mutableBigInteger.normalize();
        if (numberOfLeadingZeros > 0) {
            return n2 % i;
        }
        return n2;
    }
    
    MutableBigInteger divide(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2) {
        return this.divide(mutableBigInteger, mutableBigInteger2, true);
    }
    
    MutableBigInteger divide(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2, boolean b) {
        if (mutableBigInteger.intLen < 80 || this.intLen - mutableBigInteger.intLen < 40) {
            return this.divideKnuth(mutableBigInteger, mutableBigInteger2, b);
        }
        return this.divideAndRemainderBurnikelZiegler(mutableBigInteger, mutableBigInteger2);
    }
    
    MutableBigInteger divideKnuth(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2) {
        return this.divideKnuth(mutableBigInteger, mutableBigInteger2, true);
    }
    
    MutableBigInteger divideKnuth(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2, boolean b) {
        if (mutableBigInteger.intLen == 0) {
            throw new ArithmeticException("BigInteger divide by zero");
        }
        if (this.intLen == 0) {
            int n2 = 0;
            mutableBigInteger2.offset = n2;
            mutableBigInteger2.intLen = n2;
            return b ? new MutableBigInteger() : null;
        }
        int compare = this.compare(mutableBigInteger);
        if (compare < 0) {
            int n = 0;
            mutableBigInteger2.offset = n;
            mutableBigInteger2.intLen = n;
            return b ? new MutableBigInteger(this) : null;
        }
        if (compare == 0) {
            mutableBigInteger2.value[0] = (mutableBigInteger2.intLen = 1);
            mutableBigInteger2.offset = 0;
            return b ? new MutableBigInteger() : null;
        }
        mutableBigInteger2.clear();
        if (mutableBigInteger.intLen == 1) {
            int divideOneWord = this.divideOneWord(mutableBigInteger.value[mutableBigInteger.offset], mutableBigInteger2);
            if (b) {
                if (divideOneWord == 0) {
                    return new MutableBigInteger();
                }
                return new MutableBigInteger(divideOneWord);
            }
            else {
                return null;
            }
        }
        else {
            if (this.intLen >= 6) {
                int min = Math.min(this.getLowestSetBit(), mutableBigInteger.getLowestSetBit());
                if (min >= 96) {
                    MutableBigInteger mutableBigInteger3 = new MutableBigInteger(this);
                    mutableBigInteger = new MutableBigInteger(mutableBigInteger);
                    mutableBigInteger3.rightShift(min);
                    mutableBigInteger.rightShift(min);
                    MutableBigInteger divideKnuth = mutableBigInteger3.divideKnuth(mutableBigInteger, mutableBigInteger2);
                    divideKnuth.leftShift(min);
                    return divideKnuth;
                }
            }
            return this.divideMagnitude(mutableBigInteger, mutableBigInteger2, b);
        }
    }
    
    MutableBigInteger divideAndRemainderBurnikelZiegler(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2) {
        int intLen = this.intLen;
        int intLen2 = mutableBigInteger.intLen;
        int n = 0;
        mutableBigInteger2.intLen = n;
        mutableBigInteger2.offset = n;
        if (intLen < intLen2) {
            return this;
        }
        int n2 = 1 << 32 - Integer.numberOfLeadingZeros(intLen2 / 80);
        int n3 = (intLen2 + n2 - 1) / n2 * n2;
        long n4 = 32L * (long)n3;
        int n5 = (int)Math.max(0L, n4 - mutableBigInteger.bitLength());
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(mutableBigInteger);
        mutableBigInteger3.safeLeftShift(n5);
        MutableBigInteger mutableBigInteger4 = new MutableBigInteger(this);
        mutableBigInteger4.safeLeftShift(n5);
        int n6 = (int)((mutableBigInteger4.bitLength() + n4) / n4);
        if (n6 < 2) {
            n6 = 2;
        }
        MutableBigInteger block = mutableBigInteger4.getBlock(n6 - 1, n6, n3);
        MutableBigInteger mutableBigInteger5 = mutableBigInteger4.getBlock(n6 - 2, n6, n3);
        mutableBigInteger5.addDisjoint(block, n3);
        MutableBigInteger o = new MutableBigInteger();
        for (int i = n6 - 2; i > 0; --i) {
            MutableBigInteger divide2n1n = mutableBigInteger5.divide2n1n(mutableBigInteger3, o);
            mutableBigInteger5 = mutableBigInteger4.getBlock(i - 1, n6, n3);
            mutableBigInteger5.addDisjoint(divide2n1n, n3);
            mutableBigInteger2.addShifted(o, i * n3);
        }
        MutableBigInteger divide2n1n2 = mutableBigInteger5.divide2n1n(mutableBigInteger3, o);
        mutableBigInteger2.add(o);
        divide2n1n2.rightShift(n5);
        return divide2n1n2;
    }
    
    private MutableBigInteger divide2n1n(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2) {
        int intLen = mutableBigInteger.intLen;
        if (intLen % 2 != 0 || intLen < 80) {
            return this.divideKnuth(mutableBigInteger, mutableBigInteger2);
        }
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(this);
        mutableBigInteger3.safeRightShift(32 * (intLen / 2));
        this.keepLower(intLen / 2);
        MutableBigInteger mutableBigInteger4 = new MutableBigInteger();
        this.addDisjoint(mutableBigInteger3.divide3n2n(mutableBigInteger, mutableBigInteger4), intLen / 2);
        MutableBigInteger divide3n2n = this.divide3n2n(mutableBigInteger, mutableBigInteger2);
        mutableBigInteger2.addDisjoint(mutableBigInteger4, intLen / 2);
        return divide3n2n;
    }
    
    private MutableBigInteger divide3n2n(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2) {
        int n = mutableBigInteger.intLen / 2;
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(this);
        mutableBigInteger3.safeRightShift(32 * n);
        MutableBigInteger mutableBigInteger4 = new MutableBigInteger(mutableBigInteger);
        mutableBigInteger4.safeRightShift(n * 32);
        BigInteger lower = mutableBigInteger.getLower(n);
        MutableBigInteger divide2n1n;
        MutableBigInteger mutableBigInteger5;
        if (this.compareShifted(mutableBigInteger, n) < 0) {
            divide2n1n = mutableBigInteger3.divide2n1n(mutableBigInteger4, mutableBigInteger2);
            mutableBigInteger5 = new MutableBigInteger(mutableBigInteger2.toBigInteger().multiply(lower));
        }
        else {
            mutableBigInteger2.ones(n);
            mutableBigInteger3.add(mutableBigInteger4);
            mutableBigInteger4.leftShift(32 * n);
            mutableBigInteger3.subtract(mutableBigInteger4);
            divide2n1n = mutableBigInteger3;
            mutableBigInteger5 = new MutableBigInteger(lower);
            mutableBigInteger5.leftShift(32 * n);
            mutableBigInteger5.subtract(new MutableBigInteger(lower));
        }
        divide2n1n.leftShift(32 * n);
        divide2n1n.addLower(this, n);
        while (divide2n1n.compare(mutableBigInteger5) < 0) {
            divide2n1n.add(mutableBigInteger);
            mutableBigInteger2.subtract(MutableBigInteger.ONE);
        }
        divide2n1n.subtract(mutableBigInteger5);
        return divide2n1n;
    }
    
    private MutableBigInteger getBlock(int n, int n2, int n3) {
        int n4 = n * n3;
        if (n4 >= this.intLen) {
            return new MutableBigInteger();
        }
        int intLen;
        if (n == n2 - 1) {
            intLen = this.intLen;
        }
        else {
            intLen = (n + 1) * n3;
        }
        if (intLen > this.intLen) {
            return new MutableBigInteger();
        }
        return new MutableBigInteger(Arrays.copyOfRange(this.value, this.offset + this.intLen - intLen, this.offset + this.intLen - n4));
    }
    
    long bitLength() {
        if (this.intLen == 0) {
            return 0L;
        }
        return (long)this.intLen * 32L - (long)Integer.numberOfLeadingZeros(this.value[this.offset]);
    }
    
    long divide(long n, MutableBigInteger mutableBigInteger) {
        if (n == 0L) {
            throw new ArithmeticException("BigInteger divide by zero");
        }
        if (this.intLen == 0) {
            int n3 = 0;
            mutableBigInteger.offset = n3;
            mutableBigInteger.intLen = n3;
            return 0L;
        }
        if (n < 0L) {
            n = -n;
        }
        int n2 = (int)(n >>> 32);
        mutableBigInteger.clear();
        if (n2 == 0) {
            return (long)this.divideOneWord((int)n, mutableBigInteger) & 0xFFFFFFFFL;
        }
        return this.divideLongMagnitude(n, mutableBigInteger).toLong();
    }
    
    private static void copyAndShift(int[] array, int n, int n2, int[] array2, int n3, int n4) {
        int n5 = 32 - n4;
        int n6 = array[n];
        for (int i = 0; i < n2 - 1; ++i) {
            n6 = array[++n];
            array2[n3 + i] = (n6 << n4 | n6 >>> n5);
        }
        array2[n3 + n2 - 1] = n6 << n4;
    }
    
    private MutableBigInteger divideMagnitude(MutableBigInteger mutableBigInteger, MutableBigInteger mutableBigInteger2, boolean b) {
        int numberOfLeadingZeros = Integer.numberOfLeadingZeros(mutableBigInteger.value[mutableBigInteger.offset]);
        int intLen = mutableBigInteger.intLen;
        int[] copyOfRange;
        MutableBigInteger mutableBigInteger3;
        if (numberOfLeadingZeros > 0) {
            copyOfRange = new int[intLen];
            copyAndShift(mutableBigInteger.value, mutableBigInteger.offset, intLen, copyOfRange, 0, numberOfLeadingZeros);
            if (Integer.numberOfLeadingZeros(this.value[this.offset]) >= numberOfLeadingZeros) {
                int[] array2 = new int[this.intLen + 1];
                mutableBigInteger3 = new MutableBigInteger(array2);
                mutableBigInteger3.intLen = this.intLen;
                mutableBigInteger3.offset = 1;
                copyAndShift(this.value, this.offset, this.intLen, array2, 1, numberOfLeadingZeros);
            }
            else {
                int[] array = new int[this.intLen + 2];
                mutableBigInteger3 = new MutableBigInteger(array);
                mutableBigInteger3.intLen = this.intLen + 1;
                mutableBigInteger3.offset = 1;
                int offset = this.offset;
                int n = 0;
                int n2 = 32 - numberOfLeadingZeros;
                for (int i = 1; i < this.intLen + 1; ++i, ++offset) {
                    n = this.value[offset];
                    array[i] = (n << numberOfLeadingZeros | n >>> n2);
                }
                array[this.intLen + 1] = n << numberOfLeadingZeros;
            }
        }
        else {
            copyOfRange = Arrays.copyOfRange(mutableBigInteger.value, mutableBigInteger.offset, mutableBigInteger.offset + mutableBigInteger.intLen);
            mutableBigInteger3 = new MutableBigInteger(new int[this.intLen + 1]);
            System.arraycopy(this.value, this.offset, mutableBigInteger3.value, 1, this.intLen);
            mutableBigInteger3.intLen = this.intLen;
            mutableBigInteger3.offset = 1;
        }
        int intLen2 = mutableBigInteger3.intLen;
        int intLen3 = intLen2 - intLen + 1;
        if (mutableBigInteger2.value.length < intLen3) {
            mutableBigInteger2.value = new int[intLen3];
            mutableBigInteger2.offset = 0;
        }
        mutableBigInteger2.intLen = intLen3;
        int[] value = mutableBigInteger2.value;
        if (mutableBigInteger3.intLen == intLen2) {
            mutableBigInteger3.offset = 0;
            mutableBigInteger3.value[0] = 0;
            ++mutableBigInteger3.intLen;
        }
        int n3 = copyOfRange[0];
        long n4 = (long)n3 & 0xFFFFFFFFL;
        int n5 = copyOfRange[1];
        for (int j = 0; j < intLen3 - 1; ++j) {
            int n6 = 0;
            int n7 = mutableBigInteger3.value[j + mutableBigInteger3.offset];
            int n8 = n7 + Integer.MIN_VALUE;
            int n9 = mutableBigInteger3.value[j + 1 + mutableBigInteger3.offset];
            int n11;
            int n12;
            if (n7 == n3) {
                n11 = -1;
                n12 = n7 + n9;
                n6 = (n12 + Integer.MIN_VALUE < n8);
            }
            else {
                long n10 = (long)n7 << 32 | ((long)n9 & 0xFFFFFFFFL);
                if (n10 >= 0L) {
                    n11 = (int)(n10 / n4);
                    n12 = (int)(n10 - (long)n11 * n4);
                }
                else {
                    long divWord = divWord(n10, n3);
                    n11 = (int)(divWord & 0xFFFFFFFFL);
                    n12 = (int)(divWord >>> 32);
                }
            }
            if (n11 != 0) {
                if (n6 == 0) {
                    long n13 = (long)mutableBigInteger3.value[j + 2 + mutableBigInteger3.offset] & 0xFFFFFFFFL;
                    long n14 = ((long)n12 & 0xFFFFFFFFL) << 32 | n13;
                    long n15 = ((long)n5 & 0xFFFFFFFFL) * ((long)n11 & 0xFFFFFFFFL);
                    if (this.unsignedLongCompare(n15, n14)) {
                        --n11;
                        int n16 = (int)(((long)n12 & 0xFFFFFFFFL) + n4);
                        if (((long)n16 & 0xFFFFFFFFL) >= n4 && this.unsignedLongCompare(n15 - ((long)n5 & 0xFFFFFFFFL), ((long)n16 & 0xFFFFFFFFL) << 32 | n13)) {
                            --n11;
                        }
                    }
                }
                mutableBigInteger3.value[j + mutableBigInteger3.offset] = 0;
                if (this.mulsub(mutableBigInteger3.value, copyOfRange, n11, intLen, j + mutableBigInteger3.offset) + Integer.MIN_VALUE > n8) {
                    this.divadd(copyOfRange, mutableBigInteger3.value, j + 1 + mutableBigInteger3.offset);
                    --n11;
                }
                value[j] = n11;
            }
        }
        int n17 = 0;
        int n18 = mutableBigInteger3.value[intLen3 - 1 + mutableBigInteger3.offset];
        int n19 = n18 + Integer.MIN_VALUE;
        int n20 = mutableBigInteger3.value[intLen3 + mutableBigInteger3.offset];
        int n22;
        int n23;
        if (n18 == n3) {
            n22 = -1;
            n23 = n18 + n20;
            n17 = (n23 + Integer.MIN_VALUE < n19);
        }
        else {
            long n21 = (long)n18 << 32 | ((long)n20 & 0xFFFFFFFFL);
            if (n21 >= 0L) {
                n22 = (int)(n21 / n4);
                n23 = (int)(n21 - (long)n22 * n4);
            }
            else {
                long divWord2 = divWord(n21, n3);
                n22 = (int)(divWord2 & 0xFFFFFFFFL);
                n23 = (int)(divWord2 >>> 32);
            }
        }
        if (n22 != 0) {
            if (n17 == 0) {
                long n24 = (long)mutableBigInteger3.value[intLen3 + 1 + mutableBigInteger3.offset] & 0xFFFFFFFFL;
                long n25 = ((long)n23 & 0xFFFFFFFFL) << 32 | n24;
                long n26 = ((long)n5 & 0xFFFFFFFFL) * ((long)n22 & 0xFFFFFFFFL);
                if (this.unsignedLongCompare(n26, n25)) {
                    --n22;
                    int n27 = (int)(((long)n23 & 0xFFFFFFFFL) + n4);
                    if (((long)n27 & 0xFFFFFFFFL) >= n4 && this.unsignedLongCompare(n26 - ((long)n5 & 0xFFFFFFFFL), ((long)n27 & 0xFFFFFFFFL) << 32 | n24)) {
                        --n22;
                    }
                }
            }
            mutableBigInteger3.value[intLen3 - 1 + mutableBigInteger3.offset] = 0;
            int n28;
            if (b) {
                n28 = this.mulsub(mutableBigInteger3.value, copyOfRange, n22, intLen, intLen3 - 1 + mutableBigInteger3.offset);
            }
            else {
                n28 = this.mulsubBorrow(mutableBigInteger3.value, copyOfRange, n22, intLen, intLen3 - 1 + mutableBigInteger3.offset);
            }
            if (n28 + Integer.MIN_VALUE > n19) {
                if (b) {
                    this.divadd(copyOfRange, mutableBigInteger3.value, intLen3 - 1 + 1 + mutableBigInteger3.offset);
                }
                --n22;
            }
            value[intLen3 - 1] = n22;
        }
        if (b) {
            if (numberOfLeadingZeros > 0) {
                mutableBigInteger3.rightShift(numberOfLeadingZeros);
            }
            mutableBigInteger3.normalize();
        }
        mutableBigInteger2.normalize();
        return b ? mutableBigInteger3 : null;
    }
    
    private MutableBigInteger divideLongMagnitude(long i, MutableBigInteger mutableBigInteger) {
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger(new int[this.intLen + 1]);
        System.arraycopy(this.value, this.offset, mutableBigInteger2.value, 1, this.intLen);
        mutableBigInteger2.intLen = this.intLen;
        mutableBigInteger2.offset = 1;
        int intLen = mutableBigInteger2.intLen;
        int intLen2 = intLen - 2 + 1;
        if (mutableBigInteger.value.length < intLen2) {
            mutableBigInteger.value = new int[intLen2];
            mutableBigInteger.offset = 0;
        }
        mutableBigInteger.intLen = intLen2;
        int[] value = mutableBigInteger.value;
        int numberOfLeadingZeros = Long.numberOfLeadingZeros(i);
        if (numberOfLeadingZeros > 0) {
            i <<= numberOfLeadingZeros;
            mutableBigInteger2.leftShift(numberOfLeadingZeros);
        }
        if (mutableBigInteger2.intLen == intLen) {
            mutableBigInteger2.offset = 0;
            mutableBigInteger2.value[0] = 0;
            ++mutableBigInteger2.intLen;
        }
        int n = (int)(i >>> 32);
        long n2 = (long)n & 0xFFFFFFFFL;
        int n3 = (int)(i & 0xFFFFFFFFL);
        for (int j = 0; j < intLen2; ++j) {
            int n4 = 0;
            int n5 = mutableBigInteger2.value[j + mutableBigInteger2.offset];
            int n6 = n5 + Integer.MIN_VALUE;
            int n7 = mutableBigInteger2.value[j + 1 + mutableBigInteger2.offset];
            int n9;
            int n10;
            if (n5 == n) {
                n9 = -1;
                n10 = n5 + n7;
                n4 = (n10 + Integer.MIN_VALUE < n6);
            }
            else {
                long n8 = (long)n5 << 32 | ((long)n7 & 0xFFFFFFFFL);
                if (n8 >= 0L) {
                    n9 = (int)(n8 / n2);
                    n10 = (int)(n8 - (long)n9 * n2);
                }
                else {
                    long divWord = divWord(n8, n);
                    n9 = (int)(divWord & 0xFFFFFFFFL);
                    n10 = (int)(divWord >>> 32);
                }
            }
            if (n9 != 0) {
                if (n4 == 0) {
                    long n11 = (long)mutableBigInteger2.value[j + 2 + mutableBigInteger2.offset] & 0xFFFFFFFFL;
                    long n12 = ((long)n10 & 0xFFFFFFFFL) << 32 | n11;
                    long n13 = ((long)n3 & 0xFFFFFFFFL) * ((long)n9 & 0xFFFFFFFFL);
                    if (this.unsignedLongCompare(n13, n12)) {
                        --n9;
                        int n14 = (int)(((long)n10 & 0xFFFFFFFFL) + n2);
                        if (((long)n14 & 0xFFFFFFFFL) >= n2 && this.unsignedLongCompare(n13 - ((long)n3 & 0xFFFFFFFFL), ((long)n14 & 0xFFFFFFFFL) << 32 | n11)) {
                            --n9;
                        }
                    }
                }
                mutableBigInteger2.value[j + mutableBigInteger2.offset] = 0;
                if (this.mulsubLong(mutableBigInteger2.value, n, n3, n9, j + mutableBigInteger2.offset) + Integer.MIN_VALUE > n6) {
                    this.divaddLong(n, n3, mutableBigInteger2.value, j + 1 + mutableBigInteger2.offset);
                    --n9;
                }
                value[j] = n9;
            }
        }
        if (numberOfLeadingZeros > 0) {
            mutableBigInteger2.rightShift(numberOfLeadingZeros);
        }
        mutableBigInteger.normalize();
        mutableBigInteger2.normalize();
        return mutableBigInteger2;
    }
    
    private int divaddLong(int n, int n2, int[] array, int n3) {
        long n4 = 0L;
        array[1 + n3] = (int)(((long)n2 & 0xFFFFFFFFL) + ((long)array[1 + n3] & 0xFFFFFFFFL));
        long n5 = ((long)n & 0xFFFFFFFFL) + ((long)array[n3] & 0xFFFFFFFFL) + n4;
        array[n3] = (int)n5;
        return (int)(n5 >>> 32);
    }
    
    private int mulsubLong(int[] array, int n, int n2, int n3, int n4) {
        long n5 = (long)n3 & 0xFFFFFFFFL;
        n4 += 2;
        long n6 = ((long)n2 & 0xFFFFFFFFL) * n5;
        long n7 = (long)array[n4] - n6;
        array[n4--] = (int)n7;
        long n8 = ((long)n & 0xFFFFFFFFL) * n5 + ((n6 >>> 32) + (long)((n7 & 0xFFFFFFFFL) > ((long)~(int)n6 & 0xFFFFFFFFL)));
        long n9 = (long)array[n4] - n8;
        array[n4--] = (int)n9;
        return (int)((n8 >>> 32) + (long)((n9 & 0xFFFFFFFFL) > ((long)~(int)n8 & 0xFFFFFFFFL)));
    }
    
    private boolean unsignedLongCompare(long n, long n2) {
        return n + Long.MIN_VALUE > n2 + Long.MIN_VALUE;
    }
    
    static long divWord(long n, int n2) {
        long n3 = (long)n2 & 0xFFFFFFFFL;
        if (n3 == 1L) {
            return 0L << 32 | ((long)(int)n & 0xFFFFFFFFL);
        }
        long n4;
        long n5;
        for (n4 = (n >>> 1) / (n3 >>> 1), n5 = n - n4 * n3; n5 < 0L; n5 += n3, --n4) {}
        while (n5 >= n3) {
            n5 -= n3;
            ++n4;
        }
        return n5 << 32 | (n4 & 0xFFFFFFFFL);
    }
    
    MutableBigInteger hybridGCD(MutableBigInteger mutableBigInteger) {
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        while (mutableBigInteger.intLen != 0) {
            if (Math.abs(this.intLen - mutableBigInteger.intLen) < 2) {
                return this.binaryGCD(mutableBigInteger);
            }
            MutableBigInteger divide = this.divide(mutableBigInteger, mutableBigInteger2);
            this = mutableBigInteger;
            mutableBigInteger = divide;
        }
        return this;
    }
    
    private MutableBigInteger binaryGCD(MutableBigInteger mutableBigInteger) {
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        int lowestSetBit = this.getLowestSetBit();
        int lowestSetBit2 = mutableBigInteger.getLowestSetBit();
        lowestSetBit2 = ((lowestSetBit < lowestSetBit2) ? lowestSetBit : lowestSetBit2);
        if (lowestSetBit2 != 0) {
            this.rightShift(lowestSetBit2);
            mutableBigInteger.rightShift(lowestSetBit2);
        }
        int n = lowestSetBit2 == lowestSetBit;
        this = (n ? mutableBigInteger : this);
        int difference = n ? -1 : 1;
        int lowestSetBit3;
        while ((lowestSetBit3 = this.getLowestSetBit()) >= 0) {
            this.rightShift(lowestSetBit3);
            if (difference > 0) {
            }
            else {
                mutableBigInteger = this;
            }
            if (this.intLen < 2 && mutableBigInteger.intLen < 2) {
                mutableBigInteger2.value[0] = binaryGcd(this.value[this.offset], mutableBigInteger.value[mutableBigInteger.offset]);
                mutableBigInteger2.intLen = 1;
                mutableBigInteger2.offset = 0;
                if (lowestSetBit2 > 0) {
                    mutableBigInteger2.leftShift(lowestSetBit2);
                    return mutableBigInteger2;
                }
                return mutableBigInteger2;
            }
            else {
                if ((difference = this.difference(mutableBigInteger)) == 0) {
                    break;
                }
                this = ((difference >= 0) ? this : mutableBigInteger);
            }
        }
        if (lowestSetBit2 > 0) {
            this.leftShift(lowestSetBit2);
            return this;
        }
        return this;
    }
    
    static int binaryGcd(int i, int n) {
        if (n == 0) {
            return i;
        }
        if (i == 0) {
            return n;
        }
        int numberOfTrailingZeros = Integer.numberOfTrailingZeros(i);
        int numberOfTrailingZeros2 = Integer.numberOfTrailingZeros(n);
        i >>>= numberOfTrailingZeros;
        n >>>= numberOfTrailingZeros2;
        numberOfTrailingZeros2 = ((numberOfTrailingZeros < numberOfTrailingZeros2) ? numberOfTrailingZeros : numberOfTrailingZeros2);
        while (i != n) {
            if (i + Integer.MIN_VALUE > n + Integer.MIN_VALUE) {
                i -= n;
                i >>>= Integer.numberOfTrailingZeros(i);
            }
            else {
                n -= i;
                n >>>= Integer.numberOfTrailingZeros(n);
            }
        }
        return i << numberOfTrailingZeros2;
    }
    
    MutableBigInteger mutableModInverse(MutableBigInteger mutableBigInteger) {
        if (mutableBigInteger.isOdd()) {
            return this.modInverse(mutableBigInteger);
        }
        if (this.isEven()) {
            throw new ArithmeticException("BigInteger not invertible.");
        }
        int lowestSetBit = mutableBigInteger.getLowestSetBit();
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger(mutableBigInteger);
        mutableBigInteger2.rightShift(lowestSetBit);
        if (mutableBigInteger2.isOne()) {
            return this.modInverseMP2(lowestSetBit);
        }
        MutableBigInteger modInverse = this.modInverse(mutableBigInteger2);
        MutableBigInteger modInverseMP2 = this.modInverseMP2(lowestSetBit);
        MutableBigInteger modInverseBP2 = modInverseBP2(mutableBigInteger2, lowestSetBit);
        MutableBigInteger modInverseMP3 = mutableBigInteger2.modInverseMP2(lowestSetBit);
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger();
        MutableBigInteger mutableBigInteger4 = new MutableBigInteger();
        MutableBigInteger mutableBigInteger5 = new MutableBigInteger();
        modInverse.leftShift(lowestSetBit);
        modInverse.multiply(modInverseBP2, mutableBigInteger5);
        modInverseMP2.multiply(mutableBigInteger2, mutableBigInteger3);
        mutableBigInteger3.multiply(modInverseMP3, mutableBigInteger4);
        mutableBigInteger5.add(mutableBigInteger4);
        return mutableBigInteger5.divide(mutableBigInteger, mutableBigInteger3);
    }
    
    MutableBigInteger modInverseMP2(int n) {
        if (this.isEven()) {
            throw new ArithmeticException("Non-invertible. (GCD != 1)");
        }
        if (n > 64) {
            return this.euclidModInverse(n);
        }
        int inverseMod32 = inverseMod32(this.value[this.offset + this.intLen - 1]);
        if (n < 33) {
            return new MutableBigInteger((n == 32) ? inverseMod32 : (inverseMod32 & (1 << n) - 1));
        }
        long n2 = (long)this.value[this.offset + this.intLen - 1] & 0xFFFFFFFFL;
        if (this.intLen > 1) {
            n2 |= (long)this.value[this.offset + this.intLen - 2] << 32;
        }
        long n3 = (long)inverseMod32 & 0xFFFFFFFFL;
        long n4 = n3 * (2L - n2 * n3);
        long n5 = (n == 64) ? n4 : (n4 & (1L << n) - 1L);
        MutableBigInteger o = new MutableBigInteger(new int[2]);
        o.value[0] = (int)(n5 >>> 32);
        o.value[1] = (int)n5;
        o.intLen = 2;
        o.normalize();
        return o;
    }
    
    static int inverseMod32(int n) {
        int n2 = n * (2 - n * n);
        int n3 = n2 * (2 - n * n2);
        int n4 = n3 * (2 - n * n3);
        return n4 * (2 - n * n4);
    }
    
    static long inverseMod64(long n) {
        long n2 = n * (2L - n * n);
        long n3 = n2 * (2L - n * n2);
        long n4 = n3 * (2L - n * n3);
        long n5 = n4 * (2L - n * n4);
        long n6 = n5 * (2L - n * n5);
        assert n6 * n == 1L;
        return n6;
    }
    
    static MutableBigInteger modInverseBP2(MutableBigInteger mutableBigInteger, int n) {
        return fixup(new MutableBigInteger(1), new MutableBigInteger(mutableBigInteger), n);
    }
    
    private MutableBigInteger modInverse(MutableBigInteger mutableBigInteger) {
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger(mutableBigInteger);
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(this);
        MutableBigInteger mutableBigInteger4 = new MutableBigInteger(mutableBigInteger2);
        SignedMutableBigInteger signedMutableBigInteger = new SignedMutableBigInteger(1);
        SignedMutableBigInteger signedMutableBigInteger2 = new SignedMutableBigInteger();
        int n = 0;
        if (mutableBigInteger3.isEven()) {
            int lowestSetBit = mutableBigInteger3.getLowestSetBit();
            mutableBigInteger3.rightShift(lowestSetBit);
            signedMutableBigInteger2.leftShift(lowestSetBit);
            n = lowestSetBit;
        }
        while (!mutableBigInteger3.isOne()) {
            if (mutableBigInteger3.isZero()) {
                throw new ArithmeticException("BigInteger not invertible.");
            }
            if (mutableBigInteger3.compare(mutableBigInteger4) < 0) {
                mutableBigInteger3 = (mutableBigInteger4 = mutableBigInteger4);
                signedMutableBigInteger2 = (signedMutableBigInteger = signedMutableBigInteger);
            }
            if (((mutableBigInteger3.value[mutableBigInteger3.offset + mutableBigInteger3.intLen - 1] ^ mutableBigInteger4.value[mutableBigInteger4.offset + mutableBigInteger4.intLen - 1]) & 0x3) == 0x0) {
                mutableBigInteger3.subtract(mutableBigInteger4);
                signedMutableBigInteger.signedSubtract(signedMutableBigInteger2);
            }
            else {
                mutableBigInteger3.add(mutableBigInteger4);
                signedMutableBigInteger.signedAdd(signedMutableBigInteger2);
            }
            int lowestSetBit2 = mutableBigInteger3.getLowestSetBit();
            mutableBigInteger3.rightShift(lowestSetBit2);
            signedMutableBigInteger2.leftShift(lowestSetBit2);
            n += lowestSetBit2;
        }
        if (signedMutableBigInteger.compare(mutableBigInteger2) >= 0) {
            signedMutableBigInteger.copyValue(signedMutableBigInteger.divide(mutableBigInteger2, new MutableBigInteger()));
        }
        if (signedMutableBigInteger.sign < 0) {
            signedMutableBigInteger.signedAdd(mutableBigInteger2);
        }
        return fixup(signedMutableBigInteger, mutableBigInteger2, n);
    }
    
    static MutableBigInteger fixup(MutableBigInteger divide, MutableBigInteger mutableBigInteger, int n) {
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        int n2 = -inverseMod32(mutableBigInteger.value[mutableBigInteger.offset + mutableBigInteger.intLen - 1]);
        MutableBigInteger mutableBigInteger3;
        for (int i = 0; i < n >> 5; mutableBigInteger3 = divide, --mutableBigInteger3.intLen, ++i) {
            mutableBigInteger.mul(n2 * divide.value[divide.offset + divide.intLen - 1], mutableBigInteger2);
            divide.add(mutableBigInteger2);
        }
        int n3 = n & 0x1F;
        if (n3 != 0) {
            mutableBigInteger.mul(n2 * divide.value[divide.offset + divide.intLen - 1] & (1 << n3) - 1, mutableBigInteger2);
            divide.add(mutableBigInteger2);
            divide.rightShift(n3);
        }
        if (divide.compare(mutableBigInteger) >= 0) {
            divide = divide.divide(mutableBigInteger, new MutableBigInteger());
            return divide;
        }
        return divide;
    }
    
    MutableBigInteger euclidModInverse(int n) {
        MutableBigInteger mutableBigInteger = new MutableBigInteger(1);
        mutableBigInteger.leftShift(n);
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger(mutableBigInteger);
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(this);
        MutableBigInteger mutableBigInteger4 = new MutableBigInteger();
        MutableBigInteger divide = mutableBigInteger.divide(mutableBigInteger3, mutableBigInteger4);
        MutableBigInteger mutableBigInteger5 = divide;
        MutableBigInteger mutableBigInteger6 = new MutableBigInteger(mutableBigInteger4);
        MutableBigInteger mutableBigInteger7 = new MutableBigInteger(1);
        MutableBigInteger mutableBigInteger8 = new MutableBigInteger();
        while (!mutableBigInteger5.isOne()) {
            MutableBigInteger divide2 = mutableBigInteger3.divide(mutableBigInteger5, mutableBigInteger4);
            if (divide2.intLen == 0) {
                throw new ArithmeticException("BigInteger not invertible.");
            }
            divide2 = (mutableBigInteger3 = divide2);
            if (mutableBigInteger4.intLen == 1) {
                mutableBigInteger6.mul(mutableBigInteger4.value[mutableBigInteger4.offset], mutableBigInteger8);
            }
            else {
                mutableBigInteger4.multiply(mutableBigInteger6, mutableBigInteger8);
            }
            MutableBigInteger mutableBigInteger9 = mutableBigInteger8;
            mutableBigInteger7.add(mutableBigInteger9);
            if (mutableBigInteger3.isOne()) {
                return mutableBigInteger7;
            }
            MutableBigInteger divide3 = mutableBigInteger5.divide(mutableBigInteger3, mutableBigInteger9);
            if (divide3.intLen == 0) {
                throw new ArithmeticException("BigInteger not invertible.");
            }
            mutableBigInteger5 = divide3;
            if (mutableBigInteger9.intLen == 1) {
                mutableBigInteger7.mul(mutableBigInteger9.value[mutableBigInteger9.offset], mutableBigInteger4);
            }
            else {
                mutableBigInteger9.multiply(mutableBigInteger7, mutableBigInteger4);
            }
            mutableBigInteger8 = mutableBigInteger9;
            mutableBigInteger6.add(mutableBigInteger4);
        }
        mutableBigInteger2.subtract(mutableBigInteger6);
        return mutableBigInteger2;
    }
    
    static {
        ONE = new MutableBigInteger(1);
    }
}
